use mime::Mime;

use super::routes::Body;

type HeaderMap = Vec<String>;

#[derive(Debug)]
pub enum Outcome<S, E, F> {
    Success(S),
    Failure(E),
    Forward(F),
}

pub trait ResponseBody {
    fn get_data(&self) -> Vec<u8>;
    fn get_mime(&self) -> Mime;
    fn get_len(&self) -> usize;
}

impl ResponseBody for Body {
    fn get_data(&self) -> Vec<u8> {
        self.body.clone()
    }
    fn get_mime(&self) -> Mime {
        mime::TEXT_PLAIN
    }

    fn get_len(&self) -> usize {
        self.get_data().len()
    }
}

impl ResponseBody for &str {
    fn get_data(&self) -> Vec<u8> {
        self.as_bytes().to_vec()
    }

    fn get_mime(&self) -> Mime {
        mime::TEXT_PLAIN
    }

    fn get_len(&self) -> usize {
        self.len()
    }
}

impl ResponseBody for String {
    fn get_data(&self) -> Vec<u8> {
        self.as_bytes().to_vec()
    }

    fn get_mime(&self) -> Mime {
        mime::TEXT_PLAIN
    }

    fn get_len(&self) -> usize {
        self.len()
    }
}

pub struct Response {
    pub headers: HeaderMap,
    pub status: Option<Status>,
    pub body: Box<dyn ResponseBody>,
}

#[derive(Debug)]
pub struct Status {
    pub code: u16,
}
